@@ -88,7 +88,9 @@
 # endif
 #else
 # define OS_MAC 0
-# define OS_WIN 0
+# ifndef OS_WIN
+#  define OS_WIN 0
+# endif
 #endif
 
 /*
@@ -101,9 +103,11 @@
 # include <unistd.h>
   typedef struct OsFile OsFile;
   struct OsFile {
-    struct lockInfo *pLock;  /* Information about locks on this inode */
-    int fd;                  /* The file descriptor */
-    int locked;              /* True if this user holds the lock */
+    struct openCnt *pOpen;    /* Info about all open fd's on this inode */
+    struct lockInfo *pLock;   /* Info about locks on this inode */
+    int fd;                   /* The file descriptor */
+    int locked;               /* True if this instance holds the lock */
+    int dirfd;                /* File descriptor for the directory */
   };
 # define SQLITE_TEMPNAME_SIZE 200
 # if defined(HAVE_USLEEP) && HAVE_USLEEP
@@ -124,7 +128,12 @@
 # if defined(_MSC_VER) || defined(__BORLANDC__)
     typedef __int64 off_t;
 # else
-    typedef long long off_t;
+#  if !defined(_CYGWIN_TYPES_H)
+     typedef long long off_t;
+#    if defined(__MINGW32__)
+#      define	_OFF_T_
+#    endif
+#  endif
 # endif
 # define SQLITE_TEMPNAME_SIZE (MAX_PATH+50)
 # define SQLITE_MIN_SLEEP_MS 1
@@ -156,6 +165,7 @@ int sqliteOsFileRename(const char*, const char*);
 int sqliteOsOpenReadWrite(const char*, OsFile*, int*);
 int sqliteOsOpenExclusive(const char*, OsFile*, int);
 int sqliteOsOpenReadOnly(const char*, OsFile*);
+int sqliteOsOpenDirectory(const char*, OsFile*);
 int sqliteOsTempFileName(char*);
 int sqliteOsClose(OsFile*);
 int sqliteOsRead(OsFile*, void*, int amt);
@@ -169,6 +179,7 @@ int sqliteOsWriteLock(OsFile*);
 int sqliteOsUnlock(OsFile*);
 int sqliteOsRandomSeed(char*);
 int sqliteOsSleep(int ms);
+int sqliteOsCurrentTime(double*);
 void sqliteOsEnterMutex(void);
 void sqliteOsLeaveMutex(void);
 char *sqliteOsFullPathname(const char*);
