@@ -1,7 +1,11 @@
 /*
 ** The "printf" code that follows dates from the 1980's.  It is in
 ** the public domain.  The original comments are included here for
-** completeness.  They are slightly out-of-date.
+** completeness.  They are very out-of-date but might be useful as
+** an historical reference.  Most of the "enhancements" have been backed
+** out so that the functionality is now the same as standard printf().
+**
+**************************************************************************
 **
 ** The following modules is an enhanced replacement for the "printf" subroutines
 ** found in the standard C library.  The following enhancements are
@@ -59,65 +63,72 @@
 ** Conversion types fall into various categories as defined by the
 ** following enumeration.
 */
-enum et_type {    /* The type of the format field */
-   etRADIX,            /* Integer types.  %d, %x, %o, and so forth */
-   etFLOAT,            /* Floating point.  %f */
-   etEXP,              /* Exponentional notation. %e and %E */
-   etGENERIC,          /* Floating or exponential, depending on exponent. %g */
-   etSIZE,             /* Return number of characters processed so far. %n */
-   etSTRING,           /* Strings. %s */
-   etDYNSTRING,        /* Dynamically allocated strings. %z */
-   etPERCENT,          /* Percent symbol. %% */
-   etCHARX,            /* Characters. %c */
-   etERROR,            /* Used to indicate no such conversion type */
+#define etRADIX       1 /* Integer types.  %d, %x, %o, and so forth */
+#define etFLOAT       2 /* Floating point.  %f */
+#define etEXP         3 /* Exponentional notation. %e and %E */
+#define etGENERIC     4 /* Floating or exponential, depending on exponent. %g */
+#define etSIZE        5 /* Return number of characters processed so far. %n */
+#define etSTRING      6 /* Strings. %s */
+#define etDYNSTRING   7 /* Dynamically allocated strings. %z */
+#define etPERCENT     8 /* Percent symbol. %% */
+#define etCHARX       9 /* Characters. %c */
+#define etERROR      10 /* Used to indicate no such conversion type */
 /* The rest are extensions, not normally found in printf() */
-   etCHARLIT,          /* Literal characters.  %' */
-   etSQLESCAPE,        /* Strings with '\'' doubled.  %q */
-   etSQLESCAPE2,       /* Strings with '\'' doubled and enclosed in '',
+#define etCHARLIT    11 /* Literal characters.  %' */
+#define etSQLESCAPE  12 /* Strings with '\'' doubled.  %q */
+#define etSQLESCAPE2 13 /* Strings with '\'' doubled and enclosed in '',
                           NULL pointers replaced by SQL NULL.  %Q */
-   etORDINAL           /* 1st, 2nd, 3rd and so forth */
-};
+
+
+/*
+** An "etByte" is an 8-bit unsigned value.
+*/
+typedef unsigned char etByte;
 
 /*
 ** Each builtin conversion character (ex: the 'd' in "%d") is described
 ** by an instance of the following structure
 */
 typedef struct et_info {   /* Information about each format field */
-  int  fmttype;              /* The format field code letter */
-  int  base;                 /* The base for radix conversion */
-  char *charset;             /* The character set for conversion */
-  int  flag_signed;          /* Is the quantity signed? */
-  char *prefix;              /* Prefix on non-zero values in alt format */
-  enum et_type type;          /* Conversion paradigm */
+  char fmttype;            /* The format field code letter */
+  etByte base;             /* The base for radix conversion */
+  etByte flags;            /* One or more of FLAG_ constants below */
+  etByte type;             /* Conversion paradigm */
+  char *charset;           /* The character set for conversion */
+  char *prefix;            /* Prefix on non-zero values in alt format */
 } et_info;
 
+/*
+** Allowed values for et_info.flags
+*/
+#define FLAG_SIGNED  1     /* True if the value to convert is signed */
+#define FLAG_INTERN  2     /* True if for internal use only */
+
+
 /*
 ** The following table is searched linearly, so it is good to put the
 ** most frequently used conversion types first.
 */
 static et_info fmtinfo[] = {
-  { 'd',  10,  "0123456789",       1,    0, etRADIX,      },
-  { 's',   0,  0,                  0,    0, etSTRING,     }, 
-  { 'z',   0,  0,                  0,    0, etDYNSTRING,  }, 
-  { 'q',   0,  0,                  0,    0, etSQLESCAPE,  },
-  { 'Q',   0,  0,                  0,    0, etSQLESCAPE2, },
-  { 'c',   0,  0,                  0,    0, etCHARX,      },
-  { 'o',   8,  "01234567",         0,  "0", etRADIX,      },
-  { 'u',  10,  "0123456789",       0,    0, etRADIX,      },
-  { 'x',  16,  "0123456789abcdef", 0, "x0", etRADIX,      },
-  { 'X',  16,  "0123456789ABCDEF", 0, "X0", etRADIX,      },
-  { 'r',  10,  "0123456789",       0,    0, etORDINAL,    },
-  { 'f',   0,  0,                  1,    0, etFLOAT,      },
-  { 'e',   0,  "e",                1,    0, etEXP,        },
-  { 'E',   0,  "E",                1,    0, etEXP,        },
-  { 'g',   0,  "e",                1,    0, etGENERIC,    },
-  { 'G',   0,  "E",                1,    0, etGENERIC,    },
-  { 'i',  10,  "0123456789",       1,    0, etRADIX,      },
-  { 'n',   0,  0,                  0,    0, etSIZE,       },
-  { '%',   0,  0,                  0,    0, etPERCENT,    },
-  { 'b',   2,  "01",               0, "b0", etRADIX,      }, /* Binary */
-  { 'p',  10,  "0123456789",       0,    0, etRADIX,      }, /* Pointers */
-  { '\'',  0,  0,                  0,    0, etCHARLIT,    }, /* Literal char */
+  {  'd', 10, 1, etRADIX,      "0123456789",       0    },
+  {  's',  0, 0, etSTRING,     0,                  0    },
+  {  'z',  0, 2, etDYNSTRING,  0,                  0    },
+  {  'q',  0, 0, etSQLESCAPE,  0,                  0    },
+  {  'Q',  0, 0, etSQLESCAPE2, 0,                  0    },
+  {  'c',  0, 0, etCHARX,      0,                  0    },
+  {  'o',  8, 0, etRADIX,      "01234567",         "0"  },
+  {  'u', 10, 0, etRADIX,      "0123456789",       0    },
+  {  'x', 16, 0, etRADIX,      "0123456789abcdef", "x0" },
+  {  'X', 16, 0, etRADIX,      "0123456789ABCDEF", "X0" },
+  {  'f',  0, 1, etFLOAT,      0,                  0    },
+  {  'e',  0, 1, etEXP,        "e",                0    },
+  {  'E',  0, 1, etEXP,        "E",                0    },
+  {  'g',  0, 1, etGENERIC,    "e",                0    },
+  {  'G',  0, 1, etGENERIC,    "E",                0    },
+  {  'i', 10, 1, etRADIX,      "0123456789",       0    },
+  {  'n',  0, 0, etSIZE,       0,                  0    },
+  {  '%',  0, 0, etPERCENT,    0,                  0    },
+  {  'p', 10, 0, etRADIX,      "0123456789",       0    },
 };
 #define etNINFO  (sizeof(fmtinfo)/sizeof(fmtinfo[0]))
 
@@ -183,50 +194,47 @@ static int et_getdigit(LONGDOUBLE_TYPE *val, int *cnt){
 static int vxprintf(
   void (*func)(void*,char*,int),
   void *arg,
-  const char *format,
+  const char *fmt,
   va_list ap
 ){
-  register const char *fmt; /* The format string. */
-  register int c;           /* Next character in the format string */
-  register char *bufpt;     /* Pointer to the conversion buffer */
-  register int  precision;  /* Precision of the current field */
-  register int  length;     /* Length of the field */
-  register int  idx;        /* A general purpose loop counter */
-  int count;                /* Total number of characters output */
-  int width;                /* Width of the current field */
-  int flag_leftjustify;     /* True if "-" flag is present */
-  int flag_plussign;        /* True if "+" flag is present */
-  int flag_blanksign;       /* True if " " flag is present */
-  int flag_alternateform;   /* True if "#" flag is present */
-  int flag_zeropad;         /* True if field width constant starts with zero */
-  int flag_long;            /* True if "l" flag is present */
-  int flag_center;          /* True if "=" flag is present */
-  unsigned long longvalue;  /* Value for integer types */
+  int c;                     /* Next character in the format string */
+  char *bufpt;               /* Pointer to the conversion buffer */
+  int precision;             /* Precision of the current field */
+  int length;                /* Length of the field */
+  int idx;                   /* A general purpose loop counter */
+  int count;                 /* Total number of characters output */
+  int width;                 /* Width of the current field */
+  etByte flag_leftjustify;   /* True if "-" flag is present */
+  etByte flag_plussign;      /* True if "+" flag is present */
+  etByte flag_blanksign;     /* True if " " flag is present */
+  etByte flag_alternateform; /* True if "#" flag is present */
+  etByte flag_zeropad;       /* True if field width constant starts with zero */
+  etByte flag_long;          /* True if "l" flag is present */
+  unsigned long longvalue;   /* Value for integer types */
   LONGDOUBLE_TYPE realvalue; /* Value for real types */
-  et_info *infop;           /* Pointer to the appropriate info structure */
-  char buf[etBUFSIZE];      /* Conversion buffer */
-  char prefix;              /* Prefix character.  "+" or "-" or " " or '\0'. */
-  int  errorflag = 0;       /* True if an error is encountered */
-  enum et_type xtype;       /* Conversion paradigm */
-  char *zExtra;             /* Extra memory used for etTCLESCAPE conversions */
+  et_info *infop;            /* Pointer to the appropriate info structure */
+  char buf[etBUFSIZE];       /* Conversion buffer */
+  char prefix;               /* Prefix character.  "+" or "-" or " " or '\0'. */
+  etByte errorflag = 0;      /* True if an error is encountered */
+  etByte xtype;              /* Conversion paradigm */
+  char *zExtra;              /* Extra memory used for etTCLESCAPE conversions */
   static char spaces[] = "                                                  "
      "                                                                      ";
 #define etSPACESIZE (sizeof(spaces)-1)
 #ifndef etNOFLOATINGPOINT
-  int  exp;                 /* exponent of real numbers */
-  double rounder;           /* Used for rounding floating point values */
-  int flag_dp;              /* True if decimal point should be shown */
-  int flag_rtz;             /* True if trailing zeros should be removed */
-  int flag_exp;             /* True to force display of the exponent */
-  int nsd;                  /* Number of significant digits returned */
+  int  exp;                  /* exponent of real numbers */
+  double rounder;            /* Used for rounding floating point values */
+  etByte flag_dp;            /* True if decimal point should be shown */
+  etByte flag_rtz;           /* True if trailing zeros should be removed */
+  etByte flag_exp;           /* True to force display of the exponent */
+  int nsd;                   /* Number of significant digits returned */
 #endif
 
-  fmt = format;                     /* Put in a register for speed */
   count = length = 0;
   bufpt = 0;
   for(; (c=(*fmt))!=0; ++fmt){
     if( c!='%' ){
-      register int amt;
+      int amt;
       bufpt = (char *)fmt;
       amt = 1;
       while( (c=(*++fmt))!='%' && c!=0 ) amt++;
@@ -242,7 +250,7 @@ static int vxprintf(
     }
     /* Find out what flags are present */
     flag_leftjustify = flag_plussign = flag_blanksign = 
-     flag_alternateform = flag_zeropad = flag_center = 0;
+     flag_alternateform = flag_zeropad = 0;
     do{
       switch( c ){
         case '-':   flag_leftjustify = 1;     c = 0;   break;
@@ -250,11 +258,9 @@ static int vxprintf(
         case ' ':   flag_blanksign = 1;       c = 0;   break;
         case '#':   flag_alternateform = 1;   c = 0;   break;
         case '0':   flag_zeropad = 1;         c = 0;   break;
-        case '=':   flag_center = 1;          c = 0;   break;
         default:                                       break;
       }
     }while( c==0 && (c=(*++fmt))!=0 );
-    if( flag_center ) flag_leftjustify = 0;
     /* Get the field width */
     width = 0;
     if( c=='*' ){
@@ -304,18 +310,14 @@ static int vxprintf(
     }
     /* Fetch the info entry for the field */
     infop = 0;
+    xtype = etERROR;
     for(idx=0; idx<etNINFO; idx++){
       if( c==fmtinfo[idx].fmttype ){
         infop = &fmtinfo[idx];
+        xtype = infop->type;
         break;
       }
     }
-    /* No info entry found.  It must be an error. */
-    if( infop==0 ){
-      xtype = etERROR;
-    }else{
-      xtype = infop->type;
-    }
     zExtra = 0;
 
     /*
@@ -337,7 +339,6 @@ static int vxprintf(
     **   infop                       Pointer to the appropriate info struct.
     */
     switch( xtype ){
-      case etORDINAL:
       case etRADIX:
         if( flag_long )  longvalue = va_arg(ap,long);
         else             longvalue = va_arg(ap,int);
@@ -350,7 +351,7 @@ static int vxprintf(
         ** but leave the prefix for hex. */
         if( longvalue==0 && infop->base==8 ) flag_alternateform = 0;
 #endif
-        if( infop->flag_signed ){
+        if( infop->flags & FLAG_SIGNED ){
           if( *(long*)&longvalue<0 ){
             longvalue = -*(long*)&longvalue;
             prefix = '-';
@@ -362,25 +363,6 @@ static int vxprintf(
           precision = width-(prefix!=0);
         }
         bufpt = &buf[etBUFSIZE];
-        if( xtype==etORDINAL ){
-          long a,b;
-          a = longvalue%10;
-          b = longvalue%100;
-          bufpt -= 2;
-          if( a==0 || a>3 || (b>10 && b<14) ){
-            bufpt[0] = 't';
-            bufpt[1] = 'h';
-          }else if( a==1 ){
-            bufpt[0] = 's';
-            bufpt[1] = 't';
-          }else if( a==2 ){
-            bufpt[0] = 'n';
-            bufpt[1] = 'd';
-          }else if( a==3 ){
-            bufpt[0] = 'r';
-            bufpt[1] = 'd';
-          }
-        }
         {
           register char *cset;      /* Use registers for speed */
           register int base;
@@ -609,11 +591,6 @@ static int vxprintf(
       register int nspace;
       nspace = width-length;
       if( nspace>0 ){
-        if( flag_center ){
-          nspace = nspace/2;
-          width -= nspace;
-          flag_leftjustify = 1;
-        }
         count += nspace;
         while( nspace>=etSPACESIZE ){
           (*func)(arg,spaces,etSPACESIZE);
